@@ -70,7 +70,42 @@ lbeta_asy <- function(a,b, k.max = 5, give.all = FALSE)
   lgamma(a) - logQab_asy(a,b, give.all = give.all)
 }
 
-## MM: this is unused (why ??)
+
+### FIXME ==> ../tests/qbeta-dist.R
+##            -------- ~~~~~~~~~~~~ at the end has already *several* such approximations !!! <<< FIXME
+
+##' Taylor Approximation to  log(a*B(a,b))  for  a << 1 -- derived from a --> 0
+##'
+##' for small a,  B(a,b) ~ 1/a, hence  a*B(a,b) ~ a * 1/a = 1, hence log(a*B(a,b)) ~= 0
+##'
+##' log(a * beta(a,b)) == log(a) + log(beta(a,b)) == log(a) + lbeta(a,b), but the latter suffers cancellation !!
+##' We use the Taylor series
+##' log(a * B(a,b)) = a \sum_{n=0}^\infty (\psi^{(n)}(1) - \psi^{(n)}(b)) \frac{a^n}{(n+1)!}
+##'              where \psi^{(n)} is the n-th derivative of the "psi gamma" function
+##' derived from Abramowitz & Stegun, p.259, (6.3.14) series for \psi(1+z)
+##' it's integral is \log \Gamma(z+1) = - \gamma z + \sum_{k=2}^\infty \zeta(k)/k (-z)^k   (G)
+##' and as \Gamma(z+1) = z \Gamma(z), have \log \Gamma(y) = \log(\Gamma(y+1)/y) = - \log(y) + formula (G) above.
+##'
+##' Now, \log(a \times B(a,b)) = \log(a) + \log \frac{\Gamma(a) \Gamma(b)}{\Gamma(a+b)}
+##'                            = \log(a) + \log \Gamma(a)         + \log (\Gamma(b) / \Gamma(a+b))
+##'                            = \gamma a + \pi^2/12 a^2 +O(a^3)  + \log (\Gamma(b) / \Gamma(a+b))
+##'  ...
+##'                            = a \times \sum_{n=0}^\infty .. sum above
+
+##' @title Accurate Approximation of log(a * beta(a,b)) for small a
+##' @param a, b  numeric vectors: the non-negative arguments of Beta
+##' @return
+##' @author Martin Maechler
+laBeta <- function(a,b, nT) {
+    stopifnot(length(nT) == 1, nT == as.integer(nT), nT >= 1)
+
+### ===> ../tests/qbeta-dist.R
+##       =========~~~~~~~~~~~~ at the end has already *several* such approximations !!! <<< FIXME
+
+}
+
+
+## MM: this is unused (why ??) -- should be "uniformly better" than lbeta_asy() above
 lbetaMM <- function(a,b, cutAsy = 1e-2, verbose = FALSE)
 {
   ## Purpose:  log(beta( a, b))  which also works for HUGE  a / b or b/a
@@ -158,6 +193,11 @@ lbetaIhalf <- function(a, n)
 ### -------  ~/R/D/r-devel/R/src/nmath/toms708.c
 ###                                    ~~~~~~~~~
 
+
+
+
+
+
 ## From ~/R/D/r-devel/R/src/nmath/pgamma.c :
 ##                                --------
 ##/* Compute  log(gamma(a+1))  accurately also for small a (0 < a < 0.5). */
@@ -585,6 +625,11 @@ lgamma1p_series <- function(x, k) {
 
 algdiv <- function(a,b) .Call(C_R_algdiv, as.double(a), as.double(b))
 
+bpser <- function(a,b, x, log.p=FALSE, eps = 1e-15, verbose=FALSE, warn=TRUE) { # ../src/bpser.c
+    .Call(C_R_bpser, a, b, x, eps, log.p, verbose, warn)
+    ## return( list(r = r_, err = ier_) )
+}
+
 
 
 qnormAppr <- function(p) {
@@ -613,7 +658,7 @@ qnormUappr <- function(p,
     C3 <- 0.99229
     C4 <- 0.04481
 
-    if(missing(p)) { ## have lp = log(1-p)  <==>  e^lp = 1-p  <==>  p = 1 - e^lp
+    if(missing(p)) { ## log.p unused;  lp = log(1-p)  <==>  e^lp = 1-p  <==>  p = 1 - e^lp
         p. <- -expm1(lp)
         ## swap p <--> 1-p -- so we are where approximation is better
         swap <- if(lower.tail) p. < 1/2 else p. > 1/2 # logical vector
@@ -634,12 +679,14 @@ qnormUappr <- function(p,
     R
 }
 
-qbetaAppr.1 <- function(a, p, q, y = qnormUappr(a))
+qbetaAppr.1 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE,
+                        y = qnormUappr(a, lower.tail=lower.tail, log.p=log.p))
 {
   ## Purpose: Approximate  qbeta(a, p,q) -- Abramowitz & Stegun (26.5.22)
-  ##          qbeta(.) takes this only when  p>1 & q>1
+  ##          qbeta(.) takes this only when  p>1 & q>1 : "Carter(1947), see AS 109, remark '5.'"
   ## -------------------------------------------------------------------------
-  ## Arguments: a: percentage point;  (p,q): beta parameters
+  ## Arguments: a: percentage point, only used via qnorm*(a,..);  (p,q): beta parameters
+
   r <- (y * y - 3) / 6 ## lambda
   s <- 1 / (p + p - 1)
   t <- 1 / (q + q - 1)
@@ -651,7 +698,7 @@ qbetaAppr.1 <- function(a, p, q, y = qnormUappr(a))
 qbetaAppr.3 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE, logbeta = lbeta(p,q))
 {
    ##  a=alpha
-    ## Purpose: Approximate  qbeta(a, p,q) -- for small a --
+    ## Purpose: Approximate  qbeta(a, p,q) -- for small a [log.p=TRUE: a far negative]
     ##  Inversion of   I_x(a,b) ~= x^a / (a B(a,b))
 
     ## CARE:  log(p) + logbeta == log(p * beta(p,q)) suffers from cancellation
@@ -660,7 +707,7 @@ qbetaAppr.3 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE, logbeta = lbeta(p
     ## in  ../tests/qbeta-dist.R
     ##              ============
 
-    ## log
+    ## log.a = log("a") = log(pr_{lower_tail})
     log.a <- if(lower.tail) .D_log(a, log.p=log.p) else .D_LExp(a, log.p=log.p)
 
     pmin(1, exp(log.a +  log(p) + logbeta) / p)
@@ -669,14 +716,16 @@ qbetaAppr.3 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE, logbeta = lbeta(p
 qbetaAppr.2 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE, logbeta = lbeta(p,q))
 {
     ## Purpose: Approximate  qbeta(a, p,q)
-
+    ## log(1-"a") = log(pr_{upper_tail}):
     l1ma <- if(lower.tail) .D_LExp(a, log.p=log.p) else .D_log(a, log.p=log.p)
     ## pmax(0, ....) -- needed when  (l1ma + log(q) + logbeta) / q > 0, e.g., for
-    ## e.g. qbetaAppr.2(1/4, 1/2, 1/2)
+    ## e.g. qbetaAppr.2(1/4, 1/2, 1/2) gives -0.388
     -expm1((l1ma + log(q) + logbeta) / q)
 }
 
-qbetaAppr.4 <- function(a, p, q, y = qnormUappr(a), verbose=getOption("verbose"))
+qbetaAppr.4 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE,
+                        y = qnormUappr(a, lower.tail=lower.tail, log.p=log.p),
+                        verbose=getOption("verbose"))
 {
     ## Purpose: Approximate  qbeta(a, p,q); 'a' is only used via qnorm(a,..)
     r <- q + q
@@ -688,7 +737,22 @@ qbetaAppr.4 <- function(a, p, q, y = qnormUappr(a), verbose=getOption("verbose")
     1 - 2 / (t + 1)
 }
 
-qbetaAppr <- function(a, p, q, y = qnormUappr(a), logbeta = lbeta(p,q),
+qbetaAppr.5 <- function(a, p, q, lower.tail=TRUE, log.p=FALSE,
+                        y = qnormUappr(a, lower.tail=lower.tail, log.p=log.p),
+                        logbeta = lbeta(p,q),
+                        verbose=getOption("verbose"))
+{
+    ## Purpose: Approximate  qbeta(a, p,q) for p << q , really from  p --> 0 expansion
+    ##
+    ## based on future good  laBeta() ==robust== log(p * B(p,q))
+    ##
+    ## see MM's notes (on paper!)
+}
+
+
+qbetaAppr <- function(a, p, q, lower.tail=TRUE, log.p=FALSE,
+                      y = qnormUappr(a, lower.tail=lower.tail, log.p=log.p),
+                      logbeta = lbeta(p,q),
                       verbose = getOption("verbose") && length(a) == 1)
 {
     ## Purpose: Approximate  qbeta(a, p,q) --- for  a <= 1/2
@@ -703,13 +767,11 @@ qbetaAppr <- function(a, p, q, y = qnormUappr(a), logbeta = lbeta(p,q),
     if (p > 1 && q > 1) { ## Abramowitz & Stegun(26.5.22)
         if(verbose) cat("p,q > 1: Using qbetaAppr.1(): Abramowitz-Stegun (26.5.22)\n")
         ## 'a' is not needed here, just 'y' is
-        qbetaAppr.1(,p,q,y)
+        qbetaAppr.1(,p,q, lower.tail=lower.tail, log.p=log.p, y=y)
     } else {
         if(verbose) cat("p or q <= 1 : ")
         r <- q + q
-        ## more "robustly"
-        ## t <- 1 / (9 * q)
-        ## t <- r * (1 - t + y * sqrt(t))^3
+        ## more "robust" version of   t <- 1 / (9 * q) ; t <- r * (1 - t + y * sqrt(t))^3
         st <- 1/(3 * sqrt(q))
         t <- r * (1 - st*(st + y))^3
 
@@ -717,21 +779,24 @@ qbetaAppr <- function(a, p, q, y = qnormUappr(a), logbeta = lbeta(p,q),
 
         ## vectorized in t { ==> logical vectors instead of simple if(..) .. else if(..) .. else ..
         ans <- t
-        if(any(neg <- t <= 0)) { ##- forget t(q, y)  and  y(a)
-            if(verbose)   cat("t <= 0:         appr. 2 (beta, q)\n")
-            ans[neg] <- - expm1((log1p(-a[neg]) + log(q) + logbeta) / q)
+        if(any(neg <- t <= 0)) { ## forget y(a) and t(q, y) from above; qbetaAppr.2():
+            if(verbose)   cat("t <= 0:  using qbetaAppr.2()\n")
+            ## log(1-"a") = log(pr_{upper_tail}):
+            l1ma <- if(lower.tail) .D_LExp(a[neg], log.p=log.p) else .D_log(a[neg], log.p=log.p)
+            ## FIXME -- still ? -- : "catastrophic"  in -expm1(xx) when xx > 0 ==> -expm1(xx) = 1-e^xx < 0 "nonsense"
+            ans[neg] <- - expm1((l1ma + log(q) + logbeta) / q)
         }
         if(any(!neg)) { ##else
             t <- (4 * p + r - 2) / t
             if (any(L1 <- !neg & t <= 1)) {
-                if(verbose) cat("t > 0; t' <= 1: appr. 3 (beta, p)\n")
+                if(verbose) cat("t > 0; t' <= 1: qbetaAppr.3()\n")
                 ## ans[L1] <- exp((log(a[L1] * p) + logbeta) / p)
-                ans[L1] <- qbetaAppr.3(a[L1], p, q, logbeta = logbeta)
+                ans[L1] <- qbetaAppr.3(a[L1], p, q, lower.tail=lower.tail, log.p=log.p, logbeta=logbeta)
                 ## ==> linear relationship on log-log scale:
                 ##   log(ans) = 1/p * log(a) + (log p + log beta(p,q))/p
             }
             if (any(L2 <- !neg & t > 1)) { # the other case
-                if(verbose) cat("t > 0; t' > 1 : appr. 4 = 1-2/(t'+1)\n")
+                if(verbose) cat("t > 0; t' > 1 : qbetaAppr.4() = 1-2/(t'+1)\n")
                 ans[L2] <- 1 - 2 / (t[L2] + 1)
             }
         }
@@ -740,7 +805,7 @@ qbetaAppr <- function(a, p, q, y = qnormUappr(a), logbeta = lbeta(p,q),
 }
 
 ## Hmm, this is (sometimes!) not quite equivalent to C's :
-## ex.  qbeta.R(0.5078, .01, 5) -> 2.77558e-15    but qbeta() gives 1.776357e-15
+## ex.  qbeta.R(0.5078, .01, 5) -> 2.77558e-15 but qbeta() now correctly gives 4.651e-31
 qbeta.R	 <-  function(alpha, p, q,
                       lower.tail = TRUE, log.p = FALSE,
 		      logbeta = lbeta(p,q),
