@@ -19,8 +19,9 @@ stopifnot(exprs = {
     require(sfsmisc)
 })
 
-source(system.file(package="Matrix", "test-tools-1.R", mustWork=TRUE))
-## relerrV()
+source(system.file(package="DPQ", "test-tools.R", mustWork=TRUE))
+## => showProc.time(), ...  list_() , loadList() ,  readRDS_() , save2RDS()
+relErrV <- sfsmisc::relErrV
 
 if(!dev.interactive(orNone=TRUE)) pdf("t-nonc_P-1.pdf")
 
